*! version 1.1.1  02mar2005  by Robert Jaeckle jaeckle@ifo.de
*! version 1.1.6  17jan2006  by Marc Muendler  muendler@ucsd.edu

clear
set memory 500m
set more off
capture log close

cd "$rootslct"

log using "logs/mvprobit-prepare-selct-$waglvl-pt$par-af$aff-allyrs.log", replace

* Apply BEM cntrybemgrp definitions
capture program drop countrygrp_bem
program define countrygrp_bem
  /*-----------------------
   WEU: EMU (3), OWE (6);
   OIN: OIN (5);
   CEE: CEE (2); 
   DEV: APC (1), ODV (4) and RCA (7)
   --------------------------------------------*/
  tab cntrygrp_bem
  gen str3 cntrybemgrp = "CEE" if cntrygrp_bem=="CEE"
  replace  cntrybemgrp = "DEV" if cntrygrp_bem=="APC" | cntrygrp_bem=="ODV" | cntrygrp_bem=="OTR" | cntrygrp_bem=="RCA"
  replace  cntrybemgrp = "OIN" if cntrygrp_bem=="OIN"
  replace  cntrybemgrp = "WEU" if cntrygrp_bem=="EMU" | cntrygrp_bem=="OWE"
  label var cntrybemgrp "Country group BEM"
  rename cntrybemgrp _cntrybemgrp
  encode _cntrybemgrp, gen(cntrybemgrp) label(cntrybem)
  drop _cntrybemgrp
end

foreach tlogyear in 1998 1999 2000 2001 {

local selyear = `tlogyear' - $yrdiff
global year = `tlogyear'
global tlogyear = `tlogyear'  /* for German wage setting */
global selyear  = `selyear'   /* for German wage setting */

tempfile merge_ustan merge_sec cntrgrpbem wage_diff_`tlogyear' wage_diff_`selyear' chosen_`selyear' bef_reshape_`selyear' bef_reshape_`tlogyear' addlsec_`selyear' addlsec_`tlogyear' addlfirm

*----------------------following code identical to prep-mvprobit-med-wagediff.do------------------------------------------------
*-----substitute: `tlogyear' for 2000, ${yrdiff} for 4, `selyear' for 1996 -----------------------------------------------------
*---- substitute: Xdata/data-parent-hostcntry-pt$par-af$aff-allyrs for Xdata/data-parent-hostcntry-pt$par-af$aff ---------------


/*----------------------------*/
/* External data preperation  */
/*----------------------------*/

use "$root/conc/cntry2bemgrp", clear
rename bubakey country
sort country
compress
save `cntrgrpbem'

global bef_reshape_`selyear' "`bef_reshape_`selyear''"
global bef_reshape_`tlogyear' "`bef_reshape_`tlogyear''"
global wage_diff_`selyear' "`wage_diff_`selyear''"
global wage_diff_`tlogyear' "`wage_diff_`tlogyear''"
do $dpref/#wage-diff-$waglvl.do

global addlfirm "`addlfirm'"
global addlsec_`selyear' "`addlsec_`selyear''"
global addlsec_`tlogyear' "`addlsec_`tlogyear''"
do $dpref/#addlvars.do


/*-----------*/
/* MAIN FILE */
/*-----------*/


capture confirm file "c:/temp/bem-Xdata/data-parent-hostcntry-pt$par-af$aff-allyrs.dta"
if _rc~=0 {
    use "$rootslct/Xdata/data-parent-hostcntry-pt$par-af$aff-allyrs.dta", clear
   }
else {
    use "c:/temp/bem-Xdata/data-parent-hostcntry-pt$par-af$aff-allyrs.dta", clear
    }

sort country 
merge country using `cntrgrpbem' 
tab _merge
keep if _merge==3
drop _merge

countrygrp_bem

keep firmid ustan cntrybemgrp year nace1 sec1_bem inter

count
collapse (max) ustan nace1 sec1_bem inter, by(firmid cntrybemgrp year)  /* firmid's and ustan id's one-to-one */
count
fillin firmid cntrybemgrp year
count
gen byte chosen=(_fillin~=1)
label var inter "Intern'lization (#+1)"
label var nace1 "NACE 1.1 2-digit Sector code"
label var sec1_bem "Sector codes (BEM)"

preserve
keep if chosen==1
keep firmid ustan 
sort firmid
save `merge_ustan'
restore

sort firmid
merge firmid using `merge_ustan', update
count if _merge~=3 & _merge~=4
if r(N)~=0 {
  disp as error "_merge ~=3 & ~=4 exists"
  tab _merge
  STOP;
  }
drop _merge

preserve
keep if chosen==1
keep firmid nace1 sec1_bem inter year
sort firmid year
save `merge_sec'
restore

sort firmid year
merge firmid year using `merge_sec', update 
count if _merge~=3 & _merge~=4 & _merge~=1  /* _merge==1 are missing, meaning the MNE did not exsist at this time */
if r(N)~=0 {
  disp as error "_merge ~=3 & ~=4 exists"
  tab _merge
  STOP;
  }
tab _merge chosen

drop _merge _fillin


* merge ustan information for proper years

do $dpref/#parinfo-probit.do

gen profeqtyrat_par=profits_par/equity_par
label var profeqtyrat_par "Profits per equity parent"
gen caplabrat_par = (fixdasst_par/employmt_par) / 1
label var caplabrat_par "Cap'l/lab par [Thd EUR pc]"
keep firmid ustan year cntrybemgrp chosen nace1 sec1_bem employmt_par exports_par fixdasst_par assettot_par profits_par turnover_par equity_par liab_par inter caplabrat_par profeqtyrat_par
label var ustan "Ustan ID"


* Take natural logs (parent (ustan) specific variables) 

foreach var in employmt_par exports_par fixdasst_par assettot_par profits_par turnover_par equity_par liab_par inter caplabrat_par {
  gen ln_`var'=log(`var')
  local lbl : variable label `var'
  local lbl = subinstr("`lbl'","Profits bef. tax","Profit bftax",.)
  local lbl = subinstr("`lbl'","Debt (short & long term)","Debt (shrt&lng trm)",.)
  label var ln_`var' "ln `lbl'"
  drop `var'
  }

* reshape and label for mvprobit

foreach name in employmt_par exports_par fixdasst_par assettot_par profits_par turnover_par equity_par liab_par inter caplabrat_par {
  forvalues i=`selyear'(${yrdiff})`tlogyear' {
    local lblyr = substr("`i'",-2,2)
    local lbl : variable label ln_`name'
    qui gen ln_`name'_`i'= ln_`name'
    qui replace  ln_`name'_`i'=. if year~=`i'
    bysort firmid: egen _help=max (ln_`name'_`i')
    replace ln_`name'_`i'=_help if ln_`name'_`i'==.
    label var ln_`name'_`i' "`lbl' `lblyr'"
    drop _help
    local lbl_reshp_1 : variable label ln_`name'
    }
  drop ln_`name'
  }

foreach name in profeqtyrat_par nace1 sec1_bem {
  forvalues i=`selyear'(${yrdiff})`tlogyear' {
    local lblyr = substr("`i'",-2,2)
    local lbl : variable label `name'
    gen `name'_`i'= `name'
    replace `name'_`i'=. if year~=`i'
    bysort firmid: egen _help = max(`name'_`i')
    replace `name'_`i'=_help if `name'_`i'==.
    label var `name'_`i' "`lbl' `lblyr'"
    drop _help
    local lbl_reshp_2 : variable label `name'
    }
  drop `name'
  }

replace nace1_`selyear'=nace1_`tlogyear' if nace1_`selyear'==. /* fill missing sectors */
replace nace1_`tlogyear'=nace1_`selyear' if nace1_`tlogyear'==.
replace sec1_bem_`selyear'=sec1_bem_`tlogyear' if sec1_bem_`selyear'==. /* fill missing sectors */
replace sec1_bem_`tlogyear'=sec1_bem_`selyear' if sec1_bem_`tlogyear'==.
replace ln_inter_`selyear'=0 if  ln_inter_`selyear'==. /* fill in inter==1 if no FDI  */
replace ln_inter_`tlogyear'=0 if  ln_inter_`tlogyear'==.

gen region_year= cntrybemgrp*100000+year
keep if year==`selyear' | year==`tlogyear'
drop year cntrybemgrp
duplicates drop
reshape wide chosen, i(firmid ustan) j(region_year)

local i=0
foreach name in CEE DEV OIN WEU {
  local i=`i'+10
  forvalues j=`selyear'(${yrdiff})`tlogyear' {
    local lblyr = substr("`j'",-2,2)
    rename chosen`i'`j' chosen_`name'_`j'
    label var chosen_`name'_`j' "FDI presence `name' `lblyr'"
    }
  }


* merge additional parent-level variables

sort firmid
merge firmid using ${addlfirm}, nokeep
tab _m
drop _m


* merge variables that do not vary over firmid

forvalues j=`selyear'(${yrdiff})`tlogyear' {
  local lblyr = substr("`j'",-2,2)
  cap drop _merge
  rename nace1_`j' nace1
  sort nace1 
  merge nace1 using `wage_diff_`j'', nokeep
  tab _merge
  drop _merge wagediff_CEE wagediff_DEV wagediff_OIN wagediff_WEU
  rename w_HOM w_HOM_`j'
  foreach var of varlist ln_wagediff_CEE ln_wagediff_DEV ln_wagediff_OIN ln_wagediff_WEU {
    local lblsec = substr("`var'",-3,23)
    rename `var' `var'_`j'
    label var `var'_`j' "ln Wage diff `lblsec' `lblyr'"
    }
  sort nace1
  merge nace1 using ${addlsec_`j'}, nokeep
  tab _m
  drop _m nace1 sec1_bem_`j'
  }

compress
rename firmid mneid
drop ustan
order mneid chosen_CEE_`selyear' chosen_CEE_`tlogyear' chosen_DEV_`selyear' chosen_DEV_`tlogyear' chosen_OIN_`selyear' chosen_OIN_`tlogyear' chosen_WEU_`selyear' chosen_WEU_`tlogyear' w_HOM_`selyear' ln_wagediff_CEE_`selyear' ln_wagediff_DEV_`selyear' ln_wagediff_OIN_`selyear' ln_wagediff_WEU_`selyear' union_`selyear' w_HOM_`tlogyear' ln_wagediff_CEE_`tlogyear' ln_wagediff_DEV_`tlogyear' ln_wagediff_OIN_`tlogyear' ln_wagediff_WEU_`tlogyear' union_`tlogyear' imp_CEE_`tlogyear' exp_CEE_`tlogyear' impinp_CEE_`tlogyear' imp_DEV_`tlogyear' exp_DEV_`tlogyear' impinp_DEV_`tlogyear' imp_OIN_`tlogyear' exp_OIN_`tlogyear' impinp_OIN_`tlogyear' imp_WEU_`tlogyear' exp_WEU_`tlogyear' impinp_WEU_`tlogyear' state_par_`selyear' indwest_par_`selyear' state_par_`tlogyear' indwest_par_`tlogyear' ln_employmt_par_`selyear' ln_employmt_par_`tlogyear' ln_exports_par_`selyear' ln_exports_par_`tlogyear' ln_fixdasst_par_`selyear' ln_fixdasst_par_`tlogyear' ln_assettot_par_`selyear' ln_assettot_par_`tlogyear' ln_profits_par_`selyear' ln_profits_par_`tlogyear' ln_turnover_par_`selyear' ln_turnover_par_`tlogyear' ln_equity_par_`selyear' ln_equity_par_`tlogyear' ln_liab_par_`selyear' ln_liab_par_`tlogyear' ln_inter_`selyear' ln_inter_`tlogyear' ln_caplabrat_par_`selyear' ln_caplabrat_par_`tlogyear' profeqtyrat_par_`selyear' profeqtyrat_par_`tlogyear'
sort mneid

*----------------------above code identical to prep-mvprobit-med-wagediff.do---------------------------------------------

foreach var in chosen_CEE chosen_DEV chosen_OIN chosen_WEU w_HOM ln_wagediff_CEE ln_wagediff_DEV ln_wagediff_OIN ln_wagediff_WEU ln_employmt_par ln_exports_par ln_fixdasst_par ln_assettot_par ln_profits_par ln_turnover_par ln_equity_par ln_liab_par ln_inter ln_caplabrat_par profeqtyrat_par union state_par indwest_par {
  rename `var'_`tlogyear' `var'_now
  rename `var'_`selyear'  `var'_${yrdiff}yrlg
  }
foreach var in imp_CEE exp_CEE impinp_CEE imp_DEV exp_DEV impinp_DEV imp_OIN exp_OIN impinp_OIN imp_WEU exp_WEU impinp_WEU {
  rename `var'_`tlogyear' `var'_now
  }

save "$rootslct/Xdata/mvprobit-outwfdi-ustan-`tlogyear'd$yrdiff-$waglvl-pt$par-af$aff.dta", replace

} /* close `tlogyear' loop */

log close
