*! version 1.2.3  14nov2006  by Marc Muendler  muendler@ucsd.edu

/* Varcov correction assumes that selection equations have constant */
/* Selection equation assumed to have constant as last regressor */
/* Varcov correction assumes that sureg equations do not have constant (add a regressor called ones if constant needed) */
/* Sure equation assumed to have Mills ratio as last regressor */
/* Regional wages are supposed to be first (5) regressors in SUR equations */

clear
set memory 400m
set matsize 400
set more off
capture log close

cd "$rootslct"

log using "logs/translog-selct-2stp-$waglvl-pt$par-af$aff-cf-allyrs.log", replace

*----------------------following code identical to translog-selct-2stp-cf.do----------------------------------------------------
*-----substitute: /v3fs for /v3 ------------------------------------------------------------------------------------------------
*-----substitute: now for 00, ${yrdiff}yrlg for 96, /v3fs for /v3, mneid year for mneid, currno year for currno-----------------
*-----substitute: Xdata/mneid-forbstrap-$waglvl-pt$par-af$aff-cf-allyrs for Xdata/mneid-forbstrap-$waglvl-pt$par-af$aff-cf -----

mat drop _all

local maxiter $maxiter
local usexvar $usexvar  /* probit varlist to be used for selection correction in sureg */
local maxvars 4  /* longest used probit varlist */
local surxno 15  /* number of SURE regressors per equation (excluding noFDI indicators, excluding adj. constant/ones and mills ratio) */

adopath + "$root/%stata/ado"


/* auxiliary code for Cov correction */
/*-----------------------------------*/

capture program drop Heck2eqs
program define Heck2eqs, eclass
    version 6
    args bfull VBm rho rho1 sigma touse ind seldep eqno r2dg eqname obsprb obssur uncns
    capture est post `bfull' `VBm', obs(`obsprb')
    if _rc {
        if _rc == 506 {
            di in red "estimate of rho outside the interval [-1, 1] has led to a covariance matrix"
            di in red "that is not positive definite"
            mat `VBm' = 0*`VBm'
            * mat `VBm'[1, 1] = 0*`VBm'[1..`kreg0', 1..`kreg0']
        }
        est post `bfull' `VBm', obs(`obsprb')
    }

    est scalar N = `obsprb'
    capture qui test `ind', min
    if _rc == 0 {
        est scalar chi2 = `r(chi2)'
        est scalar df_m = `r(df)'
        est scalar p    = `r(p)'
    }
    est scalar N_cens = `obsprb' - `uncns'
    est scalar N_sure = `obssur'
    est local chi2type "Wald"
    est local title2 "(Seemingly unrelated reg w/ sample selection)"
    est local title "Sure w/ Heckman selection -- two-step estimates"
    est local mills m`r2dg'
    est scalar rho = `rho'
    if `rho1' != `rho' { est scalar rho1 = `rho1' }
    est scalar sigma = `sigma'
    est scalar lambda = [`eqname']_b[lambda`r2dg']
    est scalar selambda = [`eqname']_se[lambda`r2dg']
    est local predict "heckma_p"
end

capture program drop RepostSu
program define RepostSu, eclass
    version 8
    args Vsur
    ereturn repost V = `Vsur'
end

capture program drop Display2
program define Display2
    version 6
    syntax [, Level(integer $S_level) ]

    _crcshdr
    di in smcl in gr "                                                Number of SUR obs  = " in ye %9.0f e(N_sure)
    est display, level(`level') plus
    di in smcl in gr "         rho {c |} " in ye %10.5f e(rho)
    di in smcl in gr "       sigma {c |} " in ye %10.0g e(sigma)
    di in smcl in gr "      lambda {c |} " in ye %10.0g e(lambda)  "  " %9.0g e(selambda) 
    di in smcl in gr "{hline 13}{c BT}{hline 64}"
end

#delimit ;


/* define X-equation restrictions */
/*--------------------------------*/

* constraint define  1 [sCE]cf_wHO = [sHO]cf_wCE;
  constraint define  2 [sCE]cf_wDE = [sDE]cf_wCE;
  constraint define  3 [sCE]cf_wOI = [sOI]cf_wCE;
  constraint define  4 [sCE]cf_wWE = [sWE]cf_wCE;
* constraint define  5 [sDE]cf_wHO = [sHO]cf_wDE;
  constraint define  6 [sDE]cf_wOI = [sOI]cf_wDE;
  constraint define  7 [sDE]cf_wWE = [sWE]cf_wDE;
* constraint define  8 [sOI]cf_wHO = [sHO]cf_wOI;
  constraint define  9 [sOI]cf_wWE = [sWE]cf_wOI;
* constraint define 10 [sWE]cf_wHO = [sHO]cf_wWE;
* constraint define 11 [sHO]cf_wHO + [sHO]cf_wCE + [sHO]cf_wDE + [sHO]cf_wOI + [sHO]cf_wWE = 0;
  constraint define 12 [sCE]cf_wHO + [sCE]cf_wCE + [sCE]cf_wDE + [sCE]cf_wOI + [sCE]cf_wWE = 0;
  constraint define 13 [sDE]cf_wHO + [sDE]cf_wCE + [sDE]cf_wDE + [sDE]cf_wOI + [sDE]cf_wWE = 0;
  constraint define 14 [sOI]cf_wHO + [sOI]cf_wCE + [sOI]cf_wDE + [sOI]cf_wOI + [sOI]cf_wWE = 0;
  constraint define 15 [sWE]cf_wHO + [sWE]cf_wCE + [sWE]cf_wDE + [sWE]cf_wOI + [sWE]cf_wWE = 0;

/* With 5 regions, there are (5*5-5)/2=10 symmetry restrictions in principle. 
   However, one equation drops due to further cross-equation constraints, 
   only 6 of the symmetry restrictions are needed.
   Similarly, only 4 of the 5 restrictions on wage parameter sums are needed. */

/* variable sets for probit estimation */
/*-------------------------------------*/

do $dp/!varsets.do;

*----------------------above code identical to translog-selct-2stp-cf.do---------------------------------------------;

/* data */
/*------*/

foreach usemissg in $howmany {; /* begin `usemissg' loop */

capture mkdir "$rootslct/tex/v3fs";
capture mkdir "$rootslct/estmat/v3fs";
capture mkdir "$rootslct/tex/v3fs/pt$par-af$aff-$waglvl-cf";
capture mkdir "$rootslct/estmat/v3fs/pt$par-af$aff-$waglvl-cf";
capture mkdir "$rootslct/estmat/v3fs/pt$par-af$aff-$waglvl-cf/byreg";
capture mkdir "$rootslct/estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg'";

use "$rootslct/Xdata/mneid-regdata-d$yrdiff-$waglvl-pt$par-af$aff-cf", clear;

if `usemissg' == 0 {;
  foreach reg in $regagg {;
    local r2dg = substr("`reg'",1,2);
    foreach var in s`r2dg' w`r2dg' t`r2dg' k`r2dg' y`r2dg' cf_w`r2dg' cf_t`r2dg' cf_k`r2dg' cf`r2dg' {;
      qui replace `var'=. if nofdi`r2dg'==1;
      };
    };
  };

if `usemissg' == 2 {;
   * set regressors to zero in equations where MNE is absent;
    preserve;
      foreach reg in HOM $regagg {;
        tempfile eq`reg';
        local r2dg = substr("`reg'",1,2);
        restore, preserve;
        keep mneid year y`r2dg' cf_w* cf_t* cf_k* nofdi* cf*;
        foreach eqreg in HOM $regagg {;
          local eqr2dg = substr("`eqreg'",1,2);
          foreach var in cf`eqr2dg' cf_w`eqr2dg' cf_t`eqr2dg' cf_k`eqr2dg' nofdi`eqr2dg' {;
            qui replace `var' = 0 if y`r2dg'==0; /* nofdi`r2dg'==1 */
            rename `var' `var'_eq`r2dg';
            };
          };
        order mneid year cf_w??_eq`r2dg' cf_t??_eq`r2dg' cf_k??_eq`r2dg' nofdi??_eq`r2dg' cf`r2dg'_eq`r2dg';
        keep mneid year cf_w??_eq`r2dg' cf_t??_eq`r2dg' cf_k??_eq`r2dg' nofdi??_eq`r2dg' cf`r2dg'_eq`r2dg';
        sort mneid year;
        save `eq`reg'';
        };
    restore;
    foreach reg in HOM $regagg {;
      sort mneid year;
      merge mneid year using `eq`reg'';
      qui count if _m~=3;
      if `r(N)'>0 {;
        disp as err "bad match in `reg'";
        error;
        };
      drop _m;
      };
    drop nofdiCE_eqCE nofdiDE_eqDE nofdiOI_eqOI nofdiWE_eqWE;
  }; /* close if `usemissg' == 2 */

*----------------------following code identical to translog-selct-2stp-cf.do----------------------------------------------------;
*-----substitute: now for 00, ${yrdiff}yrlg for 96, /v3fs for /v3, mneid year for mneid, currno year for currno-----------------;


/* first step: probit estimation */
/*-------------------------------*/

foreach reg in $regagg {;
  forvalues xvars = `maxvars'(-1)0 {;
    local r2dg = substr("`reg'",1,2);
    tempvar pxb;
    tempname chk;

    * probit regressions with sector dummies;
    xi: dprobit i`r2dg'now iCE${yrdiff}yrlg iDE${yrdiff}yrlg iOI${yrdiff}yrlg iWE${yrdiff}yrlg ${par_`xvars'} i.sec1, asis iterate(`maxiter');
    capture local chk = rowsof(dprobit`r2dg'_b);  /* check whether matrix exists to add to */
    if _rc==111 {;
      est2vec dprobit`r2dg', vars(iCE${yrdiff}yrlg iDE${yrdiff}yrlg iOI${yrdiff}yrlg iWE${yrdiff}yrlg ${par_`maxvars'} _Isec1_bem_1 _Isec1_bem_2 _Isec1_bem_3 _Isec1_bem_4 _Isec1_bem_5 _Isec1_bem_6 _Isec1_bem_7 _Isec1_bem_8 _Isec1_bem_9 _cons) e(N ll chi2 df_m r2_p pbar xbar) name(`r2dg'`xvars'i) altvec(dfdx se_dfdx);
      };
    else {;
      est2vec dprobit`r2dg', addto(dprobit`r2dg') name(`r2dg'`xvars'i) altvec(dfdx se_dfdx);
      };
    qui probit;  /* report all probit regressions here, only one `xvars' reported with Heckman output */
    capture local chk = rowsof(probit`r2dg'_b);
    if _rc==111 {;
      est2vec probit`r2dg', vars(iCE${yrdiff}yrlg iDE${yrdiff}yrlg iOI${yrdiff}yrlg iWE${yrdiff}yrlg ${par_`maxvars'} _Isec1_bem_1 _Isec1_bem_2 _Isec1_bem_3 _Isec1_bem_4 _Isec1_bem_5 _Isec1_bem_6 _Isec1_bem_7 _Isec1_bem_8 _Isec1_bem_9 _cons) e(N ll chi2 df_m r2_p) name(`r2dg'`xvars'i);
      est2vec probit`r2dg', r(entry_chi2 entry_df entry_p exit_chi2 exit_df exit_p hyst_chi2 hyst_df hyst_p) name(`r2dg'`xvars'i);
      };
    else {;
      est2vec probit`r2dg', addto(probit`r2dg') name(`r2dg'`xvars'i);
      est2vec probit`r2dg', raddto(probit`r2dg') name(`r2dg'`xvars'i);
      };

    * probit regressions without sector dummies (used for translog);
    * qui reg i`r2dg'now iCE${yrdiff}yrlg iDE${yrdiff}yrlg iOI${yrdiff}yrlg iWE${yrdiff}yrlg ${par_`xvars'};
    * qui predict linres`r2dg', res;
    dprobit i`r2dg'now iCE${yrdiff}yrlg iDE${yrdiff}yrlg iOI${yrdiff}yrlg iWE${yrdiff}yrlg ${par_`xvars'}, asis iterate(`maxiter');
    est2vec dprobit`r2dg', addto(dprobit`r2dg') name(`r2dg'`xvars') altvec(dfdx se_dfdx);
    qui probit;  /* report all probit regressions here, only one `xvars' reported with Heckman output */
    est2vec probit`r2dg', addto(probit`r2dg') name(`r2dg'`xvars');
    test - _cons = 0; 
    est2vec r1, r(chi2 df p) name(`r2dg'`xvars');
    test i`r2dg'${yrdiff}yrlg + _cons = 0; 
    est2vec r2, r(chi2 df p) name(`r2dg'`xvars');
    test - i`r2dg'${yrdiff}yrlg = 0; 
    est2vec r3, r(chi2 df p) name(`r2dg'`xvars');
    mat probit`r2dg'_r = probit`r2dg'_r , (r1_r \ r2_r \ r3_r);
    * matrix rownames probit`r2dg'_r = entry_chi2 entry_df entry_p exit_chi2 exit_df exit_p hyst_chi2 hyst_df hyst_p;
    mat drop r1_r r2_r r3_r;

    * predict inverse of Mills' ratios and delta from probit regression without sector dummies;
    if `xvars' == `usexvar' {;   /* Only xvars `usexvar' used for second stage */
      mat bprb`r2dg' = get(_b);
      mat Vprb`r2dg' = get(VCE);
      local obs`r2dg' = e(N);
      local uncns`r2dg' = int(e(pbar)*e(N));
      local ll`r2dg' = e(ll);
      local chi2`r2dg' = e(chi2);
      local dfm`r2dg' = e(df_m);
      local r2p`r2dg' = e(r2_p);   

      predict double `pxb', xb;
      gen double m`r2dg' = normd(`pxb') / normprob(`pxb');
      label var m`r2dg' "Inv. Mills ratio `r2dg' (xvars `xvars')";
      gen double dlta`r2dg' = m`r2dg' * (m`r2dg' + `pxb');  /* sigma_\nu = 1 in probit */
      label var dlta`r2dg' "Pred. delta `r2dg' (xvars `xvars')";
      qui summ dlta`r2dg' if e(sample) & i`r2dg'now, meanonly;
      scalar dltbar`r2dg' = r(mean);
      
      predict double zgamma`r2dg', xb;
      label var zgamma`r2dg' "Linear prediction `r2dg' (Z'gammahat)";
      };
    * else {;
    *   drop linres*;
    *   };
    };
  };
  gen byte dltaHO = .;  /* Permanent presence in Home */

  if `usemissg'==2 {;
    foreach eqreg in $regagg {;
      local eqr2dg = substr("`eqreg'",1,2);
      qui gen double m`eqr2dg'_eq`eqr2dg' = m`eqr2dg';
      replace m`eqr2dg'_eq`eqr2dg' = 0 if y`eqr2dg'==0; /* nofdi`eqr2dg'==1 */;
      };
    };

preserve;
  * drop _* zgamma* linres*;
  drop _* zgamma*;
  sort mneid year;
  save "$rootslct/Xdata/mneid-forbstrap-$waglvl-pt$par-af$aff-cf-allyrs", replace;
restore, preserve;
  * keep mneid year i??now zgamma* linres*;
  keep mneid year i??now zgamma*;
  replace mneid = _n;
  rename mneid currno;
  sort currno year;
  save "$rootslct/estmat/v3fs/pt$par-af$aff-$waglvl-cf/byreg/probit-zgamma-$waglvl-pt$par-af$aff", replace;
restore;
* drop zgamma* linres*;
drop zgamma*;

* build matrix prbdlnw of probit wage coefficients;
foreach type in b se {;  /* Build up prbdlnw columns in order of $regagg */
  foreach reg in $regagg {;
    local r2dg = substr("`reg'",1,2);
    tempname plnw`r2dg';
    mat `plnw`r2dg'' = probit`r2dg'_`type';
    matkeep `plnw`r2dg'', matrix(`plnw`r2dg'') ifc(`r2dg'`usexvar') over;
    };
  local cols : word count $regagg;
  tokenize $regagg;
  local j 1;
  local `j' = substr("``j''",1,2);
  mat prbdlnw_`type' = `plnw``j''';
  forvalues j = 2/`cols' {;
    local `j' = substr("``j''",1,2);
    mat prbdlnw_`type' = prbdlnw_`type' , `plnw``j''';
    };
  matrix colnames prbdlnw_`type' = $regagg;
  matins prbdlnw_`type', matrix(prbdlnw_`type') ifr(w_HOM w_othr_CEE w_othr_DEV w_othr_OIN w_othr_WEU) over;
  };
* do not replace missing and insignificant with 0;
* local regj = 0;
* foreach reg in $regagg {;
*   local regj = `regj' + 1;
*   local wi = 0;
*   foreach weff in HOM $regagg {;
*     local wi = `wi' + 1;
*     if prbdlnw_b[`wi',`regj'] == -999 | prbdlnw_b[`wi',`regj'] == . | prbdlnw_se[`wi',`regj'] == -999 | prbdlnw_se[`wi',`regj'] == . | abs(prbdlnw_b[`wi',`regj']/prbdlnw_se[`wi',`regj']) < abs(invnorm((100-${S_level})/200)) {;
*       mat prbdlnw_b[`wi',`regj'] = 0;
*       };
*     };
*   };
mat prbdlnw = prbdlnw_b;
mat drop prbdlnw_b prbdlnw_se;
mat prbdlnw = prbdlnw '; /* regions in rows, wage effects in columns */
local rownames : rowfullnames prbdlnw;
local colnames : colfullnames prbdlnw;
local rownum : word count `rownames';
local colnum : word count `colnames';
tokenize `rownames';
forvalues rowi = 1/`rownum' {;
  local rownew = "i_" + substr("``rowi''",-3,3);
  matname prbdlnw `rownew', rows(`rowi') explicit;
  };
tokenize `colnames';
forvalues coli = 1/`colnum' {;
  local colnew = "w_" + substr("``coli''",-3,3);
  matname prbdlnw `colnew', columns(`coli') explicit;
  };
disp in gr "Matrix of wage effects on FDI presence (usemissg: " in ye `usemissg' in gr ")";
mat list prbdlnw;


/* second step: seemingly unrelated regressions (quietly repeated below; make any changes here and there) */
/*--------------------------------------------------------------------------------------------------------*/

if `usemissg' == 0 {;
sureg (sCE: yCE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE cfCE mCE, nocons)
      (sDE: yDE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE cfDE mDE, nocons)
      (sOI: yOI = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE cfOI mOI, nocons)
      (sWE: yWE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE cfWE mWE, nocons)
      , constraints(2 3 4 6 7 9 12 13 14 15) corr isur iterate(`maxiter');
*     (sHO: yHO = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE cfHO mHO, nocons); /* left-out equation */
};

if `usemissg' == 1 {;
sureg (sCE: yCE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiOI nofdiWE cfCE mCE, nocons)
      (sDE: yDE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiOI nofdiWE cfDE mDE, nocons)
      (sOI: yOI = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiOI nofdiWE cfOI mOI, nocons)
      (sWE: yWE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiOI nofdiWE cfWE mWE, nocons)
      , constraints(2 3 4 6 7 9 12 13 14 15) corr isur iterate(`maxiter');
*     (sHO: yHO = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiOI nofdiWE cfHO mHO, nocons); /* left-out equation */
};

if `usemissg' == 2 {;
sureg (sCE: yCE = cf_wHO_eqCE cf_wCE_eqCE cf_wDE_eqCE cf_wOI_eqCE cf_wWE_eqCE cf_tHO_eqCE cf_tCE_eqCE cf_tDE_eqCE cf_tOI_eqCE cf_tWE_eqCE cf_kHO_eqCE cf_kCE_eqCE cf_kDE_eqCE cf_kOI_eqCE cf_kWE_eqCE nofdiDE_eqCE nofdiOI_eqCE nofdiWE_eqCE cfCE_eqCE mCE_eqCE, nocons)
      (sDE: yDE = cf_wHO_eqDE cf_wCE_eqDE cf_wDE_eqDE cf_wOI_eqDE cf_wWE_eqDE cf_tHO_eqDE cf_tCE_eqDE cf_tDE_eqDE cf_tOI_eqDE cf_tWE_eqDE cf_kHO_eqDE cf_kCE_eqDE cf_kDE_eqDE cf_kOI_eqDE cf_kWE_eqDE nofdiCE_eqDE nofdiOI_eqDE nofdiWE_eqDE cfDE_eqDE mDE_eqDE, nocons)
      (sOI: yOI = cf_wHO_eqOI cf_wCE_eqOI cf_wDE_eqOI cf_wOI_eqOI cf_wWE_eqOI cf_tHO_eqOI cf_tCE_eqOI cf_tDE_eqOI cf_tOI_eqOI cf_tWE_eqOI cf_kHO_eqOI cf_kCE_eqOI cf_kDE_eqOI cf_kOI_eqOI cf_kWE_eqOI nofdiCE_eqOI nofdiDE_eqOI nofdiWE_eqOI cfOI_eqOI mOI_eqOI, nocons)
      (sWE: yWE = cf_wHO_eqWE cf_wCE_eqWE cf_wDE_eqWE cf_wOI_eqWE cf_wWE_eqWE cf_tHO_eqWE cf_tCE_eqWE cf_tDE_eqWE cf_tOI_eqWE cf_tWE_eqWE cf_kHO_eqWE cf_kCE_eqWE cf_kDE_eqWE cf_kOI_eqWE cf_kWE_eqWE nofdiCE_eqWE nofdiDE_eqWE nofdiOI_eqWE cfWE_eqWE mWE_eqWE, nocons)
      , constraints(2 3 4 6 7 9 12 13 14 15) corr isur iterate(`maxiter');
*     (sHO: yHO = cf_wHO_eqHO cf_wCE_eqHO cf_wDE_eqHO cf_wOI_eqHO cf_wWE_eqHO cf_tHO_eqHO cf_tCE_eqHO cf_tDE_eqHO cf_tOI_eqHO cf_tWE_eqHO cf_kHO_eqHO cf_kCE_eqHO cf_kDE_eqHO cf_kOI_eqHO cf_kWE_eqHO nofdiCE_eqHO nofdiDE_eqHO nofdiOI_eqHO nofdiWE_eqHO cfHO_eqHO mHO_eqHO, nocons); /* left-out equation */
};

* alternative, without nofdi indicators;
* if `usemissg' == 3 {;
* sureg (sCE: yCE = cf_wHO_eqCE cf_wCE_eqCE cf_wDE_eqCE cf_wOI_eqCE cf_wWE_eqCE cf_tHO_eqCE cf_tCE_eqCE cf_tDE_eqCE cf_tOI_eqCE cf_tWE_eqCE cf_kHO_eqCE cf_kCE_eqCE cf_kDE_eqCE cf_kOI_eqCE cf_kWE_eqCE cfCE_eqCE mCE_eqCE, nocons)
*       (sDE: yDE = cf_wHO_eqDE cf_wCE_eqDE cf_wDE_eqDE cf_wOI_eqDE cf_wWE_eqDE cf_tHO_eqDE cf_tCE_eqDE cf_tDE_eqDE cf_tOI_eqDE cf_tWE_eqDE cf_kHO_eqDE cf_kCE_eqDE cf_kDE_eqDE cf_kOI_eqDE cf_kWE_eqDE cfDE_eqDE mDE_eqDE, nocons)
*       (sOI: yOI = cf_wHO_eqOI cf_wCE_eqOI cf_wDE_eqOI cf_wOI_eqOI cf_wWE_eqOI cf_tHO_eqOI cf_tCE_eqOI cf_tDE_eqOI cf_tOI_eqOI cf_tWE_eqOI cf_kHO_eqOI cf_kCE_eqOI cf_kDE_eqOI cf_kOI_eqOI cf_kWE_eqOI cfOI_eqOI mOI_eqOI, nocons)
*       (sWE: yWE = cf_wHO_eqWE cf_wCE_eqWE cf_wDE_eqWE cf_wOI_eqWE cf_wWE_eqWE cf_tHO_eqWE cf_tCE_eqWE cf_tDE_eqWE cf_tOI_eqWE cf_tWE_eqWE cf_kHO_eqWE cf_kCE_eqWE cf_kDE_eqWE cf_kOI_eqWE cf_kWE_eqWE cfWE_eqWE mWE_eqWE, nocons)
*       , constraints(2 3 4 6 7 9 12 13 14 15) corr isur iterate(`maxiter');
*       (sHO: yHO = cf_wHO_eqHO cf_wCE_eqHO cf_wDE_eqHO cf_wOI_eqHO cf_wWE_eqHO cf_tHO_eqHO cf_tCE_eqHO cf_tDE_eqHO cf_tOI_eqHO cf_tWE_eqHO cf_kHO_eqHO cf_kCE_eqHO cf_kDE_eqHO cf_kOI_eqHO cf_kWE_eqHO cfHO_eqHO mHO_eqHO, nocons); /* left-out equation */
* };

/* consistent estimates of sigma, rho and variance-covariance matrix */
/*-------------------------------------------------------------------*/

* dependent sure variables;
tokenize `e(depvar)';
local i 1;
while `i' <= e(k_eq) {;
  local depv`i' ``i'';
  local i = `i' + 1;
  };

* sigma and rho;
tokenize `e(eqnames)';
disp "`e(k_eq)' sure equations. 1:`1' 2:`2' 3:`3' 4:`4'";
local i 1;
while `i' <= e(k_eq) {; /* Correct equation by equation */
  local r2dg = substr("``i''",2,3);
  * tempname rss ebar adj ratio;
  scalar rss`i' = e(rss_`i');
  scalar ebar`i' = rss`i' / e(N);
  scalar sigma`i' = sqrt( ebar`i' + dltbar`r2dg'*([``i'']_b[m`r2dg'])^2 );
  scalar rho`i' = [``i'']_b[m`r2dg'] / sigma`i';
  scalar rho1`i' = rho`i';
  if abs(rho`i') > 1 {;
    scalar rho`i' = rho`i'/abs(rho`i');
    noi di in blue "note: two-step estimate of rho = " rho1`i' " truncated to " rho`i' " and sigma reset.";
    scalar sigma`i' = [``i'']_b[m`r2dg'] / rho`i';
  };
  if sigma`i' == 0 {; scalar sigma`i' = .001; };
  if rho`i' == . {; scalar rho`i' = 0; };
  local i = `i' + 1;
  }; 

* correct Cov equation by equation;
tokenize `e(eqnames)';
* disp "`e(k_eq)' sure equations. 1:`1' 2:`2' 3:`3' 4:`4'";
forvalues i = 1/`e(k_eq)' {;
  local reg`i' ``i'';
  };
forvalues i = 1/`e(k_eq)' {;
  local r2dg = substr("`reg`i''",2,3);
  * disp in blue "`i', `r2dg'";

  * compute Heckman's two-step consistent estimates for equation-wise partitions of Cov;
  * get X'X inverse from the second-step regression, equation by equation;
  tempname b XpX1 F rho2 Q;
  matrix `XpX1' = get(VCE);
  mat `b' = get(_b);
  local regnum = rowsof(`XpX1')/e(k_eq);
  local frm = (`i'-1)*`regnum' + 1;
  local to = `i'*`regnum';
  mat `b' = `b'[1,`frm'..`to'];
  * mat list `b';
  mat `XpX1' = `XpX1'[`frm'..`to',`frm'..`to'] / e(rmse_`i')^2;

  * Heckman's adjustment to Cov;
  tempname redVprb redXpX1; 
  mat `redVprb' = Vprb`r2dg'[1..rowsof(Vprb`r2dg')-1,1..colsof(Vprb`r2dg')-1]; /* Selection equation assumed to have constant as last regressor */
  local prbind : colnames `redVprb'; 
  mat `redXpX1' = `XpX1'[1..rowsof(`XpX1')-1,1..colsof(`XpX1')-1]; /* Sure equation assumed to have mills ratio as last regressor */
  local surind`i' : colnames `redXpX1';
  local fulnam "`surind`i'' lambda`r2dg'";
  local kreg : word count `fulnam';
  local kreg1 = `kreg' + 1;
  qui mat accum `F' = `surind`i'' m`r2dg' `prbind' [iw=dlta`r2dg'] if e(sample) & i`r2dg'now; /* Selection equation assumed to have constant */
  mat `F' = `F'[1..`kreg',`kreg1'...];
  mat rowname `F' = `fulnam';
  scalar `rho2' = rho`i' * rho`i';
  * mat list `F';
  * mat list Vprb`r2dg';
  mat `Q' = `rho2' * `F' * Vprb`r2dg' * `F'';
  * mat list `Q';

  * finish variance computation;
  tempvar rho2del;
  tempname VBm;
  qui gen double `rho2del' = 1 - `rho2' * dlta`r2dg'  if e(sample);
  qui mat accum VBm`i' = `surind`i'' m`r2dg' [iw=`rho2del'], nocons, if e(sample) & i`r2dg'now;
  mat VBm`i' = (sigma`i')^2 * `XpX1' * (VBm`i' + `Q') * `XpX1';

  * build pairs of first-and-second-step-equations by region;
  local eqnam;
  local selname i`r2dg'now;
  local kprb : word count `prbind';
  local j 1;
  tokenize `prbind';
  while `j' <= `kprb' {;
  local eqnam `eqnam' `selname':``j'';
  local j = `j' + 1;
  };
  local eqnam `eqnam' `selname':_cons;
  local kprb = `kprb' + 1;
  tokenize `fulnam';
  local j 1;
  while `j' < `kreg' {;
    local eqnam `eqnam' `depv`i'':``j'';
    local j = `j' + 1;
    };
  local eqnam `eqnam' `depv`i'':lambda`r2dg';
  * disp "`eqnam'";

  * put selection equation and sure together;
  tempname b0 bmills Vmills CovMills Vfull zeros zeroPrb;
  mat Vsur`i' = VBm`i'; /* for sure correction below */
  mat Vsur`i' = (Vsur`i' + Vsur`i'') / 2;
  local kreg0 = `kreg' - 1;
  mat `b0' = `b'[1,1..`kreg0'];
  mat `bmills' = `b'[1,`kreg'];
  mat `Vmills' = VBm`i'[`kreg',`kreg'];
  mat `CovMills' = VBm`i'[1..`kreg0',`kreg'];
  mat VBm`i' = VBm`i'[1..`kreg0',1..`kreg0'];
  mat `zeros' = J(`kreg0',`kprb',0);
  mat `zeroPrb' = J(`kprb',1,0);
  mat bfull`i' = bprb`r2dg' , `b0' , `bmills';
  mat VBm`i' = ( Vprb`r2dg' , `zeros'' , `zeroPrb' \
                 `zeros' , VBm`i' , `CovMills' \
                 `zeroPrb'' , `CovMills'' , `Vmills' );
  mat VBm`i' = (VBm`i' + VBm`i'') / 2;
  mat colnames bfull`i' = `eqnam';
  mat rownames VBm`i' = `eqnam';
  mat colnames VBm`i' = `eqnam';
};

/* report results from Heckman's two-step procedure */
tokenize `e(eqnames)';
* disp "`e(k_eq)' sure equations. 1:`1' 2:`2' 3:`3' 4:`4'";
forvalues i = 1/`e(k_eq)' {;
  local r2dg = substr("``i''",2,3);
  noi Heck2eqs bfull`i' VBm`i' rho`i' rho1`i' sigma`i' "e(sample)" "`surind`i''" i`r2dg'now `i' `r2dg' `depv`i'' `obs`r2dg'' e(N) `uncns`r2dg'';
  Display2 , level($S_level);
  capture local chk = rowsof(sure2stp_b);
  est2vec sure2stp`r2dg', e(N N_cens N_sure chi2 df_m p rho sigma lambda selambda) name(`r2dg');
};


/* re-run second step: sure posting w/ corrected VCE if positive semi-definite */
/*-----------------------------------------------------------------------------*/

if `usemissg' == 0 {;
qui sureg (sCE: yCE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE cfCE mCE, nocons)
      (sDE: yDE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE cfDE mDE, nocons)
      (sOI: yOI = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE cfOI mOI, nocons)
      (sWE: yWE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE cfWE mWE, nocons)
      , constraints(2 3 4 6 7 9 12 13 14 15) corr isur iterate(`maxiter');
*     (sHO: yHO = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE cfHO mHO, nocons); /* left-out equation */
};

if `usemissg' == 1 {;
qui sureg (sCE: yCE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiOI nofdiWE cfCE mCE, nocons)
      (sDE: yDE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiOI nofdiWE cfDE mDE, nocons)
      (sOI: yOI = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiOI nofdiWE cfOI mOI, nocons)
      (sWE: yWE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiOI nofdiWE cfWE mWE, nocons)
      , constraints(2 3 4 6 7 9 12 13 14 15) corr isur iterate(`maxiter');
*     (sHO: yHO = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiOI nofdiWE cfHO mHO, nocons); /* left-out equation */
};

if `usemissg' == 2 {;
qui sureg (sCE: yCE = cf_wHO_eqCE cf_wCE_eqCE cf_wDE_eqCE cf_wOI_eqCE cf_wWE_eqCE cf_tHO_eqCE cf_tCE_eqCE cf_tDE_eqCE cf_tOI_eqCE cf_tWE_eqCE cf_kHO_eqCE cf_kCE_eqCE cf_kDE_eqCE cf_kOI_eqCE cf_kWE_eqCE nofdiDE_eqCE nofdiOI_eqCE nofdiWE_eqCE cfCE_eqCE mCE_eqCE, nocons)
      (sDE: yDE = cf_wHO_eqDE cf_wCE_eqDE cf_wDE_eqDE cf_wOI_eqDE cf_wWE_eqDE cf_tHO_eqDE cf_tCE_eqDE cf_tDE_eqDE cf_tOI_eqDE cf_tWE_eqDE cf_kHO_eqDE cf_kCE_eqDE cf_kDE_eqDE cf_kOI_eqDE cf_kWE_eqDE nofdiCE_eqDE nofdiOI_eqDE nofdiWE_eqDE cfDE_eqDE mDE_eqDE, nocons)
      (sOI: yOI = cf_wHO_eqOI cf_wCE_eqOI cf_wDE_eqOI cf_wOI_eqOI cf_wWE_eqOI cf_tHO_eqOI cf_tCE_eqOI cf_tDE_eqOI cf_tOI_eqOI cf_tWE_eqOI cf_kHO_eqOI cf_kCE_eqOI cf_kDE_eqOI cf_kOI_eqOI cf_kWE_eqOI nofdiCE_eqOI nofdiDE_eqOI nofdiWE_eqOI cfOI_eqOI mOI_eqOI, nocons)
      (sWE: yWE = cf_wHO_eqWE cf_wCE_eqWE cf_wDE_eqWE cf_wOI_eqWE cf_wWE_eqWE cf_tHO_eqWE cf_tCE_eqWE cf_tDE_eqWE cf_tOI_eqWE cf_tWE_eqWE cf_kHO_eqWE cf_kCE_eqWE cf_kDE_eqWE cf_kOI_eqWE cf_kWE_eqWE nofdiCE_eqWE nofdiDE_eqWE nofdiOI_eqWE cfWE_eqWE mWE_eqWE, nocons)
      , constraints(2 3 4 6 7 9 12 13 14 15) corr isur iterate(`maxiter');
*     (sHO: yHO = cf_wHO_eqHO cf_wCE_eqHO cf_wDE_eqHO cf_wOI_eqHO cf_wWE_eqHO cf_tHO_eqHO cf_tCE_eqHO cf_tDE_eqHO cf_tOI_eqHO cf_tWE_eqHO cf_kHO_eqHO cf_kCE_eqHO cf_kDE_eqHO cf_kOI_eqHO cf_kWE_eqHO nofdiCE_eqHO nofdiDE_eqHO nofdiOI_eqHO nofdiWE_eqHO cfHO_eqHO mHO_eqHO, nocons); /* left-out equation */
};

* alternative, without nofdi indicators;
* if `usemissg' == 3 {;
* qui sureg (sCE: yCE = cf_wHO_eqCE cf_wCE_eqCE cf_wDE_eqCE cf_wOI_eqCE cf_wWE_eqCE cf_tHO_eqCE cf_tCE_eqCE cf_tDE_eqCE cf_tOI_eqCE cf_tWE_eqCE cf_kHO_eqCE cf_kCE_eqCE cf_kDE_eqCE cf_kOI_eqCE cf_kWE_eqCE cfCE_eqCE mCE_eqCE, nocons)
*       (sDE: yDE = cf_wHO_eqDE cf_wCE_eqDE cf_wDE_eqDE cf_wOI_eqDE cf_wWE_eqDE cf_tHO_eqDE cf_tCE_eqDE cf_tDE_eqDE cf_tOI_eqDE cf_tWE_eqDE cf_kHO_eqDE cf_kCE_eqDE cf_kDE_eqDE cf_kOI_eqDE cf_kWE_eqDE cfDE_eqDE mDE_eqDE, nocons)
*       (sOI: yOI = cf_wHO_eqOI cf_wCE_eqOI cf_wDE_eqOI cf_wOI_eqOI cf_wWE_eqOI cf_tHO_eqOI cf_tCE_eqOI cf_tDE_eqOI cf_tOI_eqOI cf_tWE_eqOI cf_kHO_eqOI cf_kCE_eqOI cf_kDE_eqOI cf_kOI_eqOI cf_kWE_eqOI cfOI_eqOI mOI_eqOI, nocons)
*       (sWE: yWE = cf_wHO_eqWE cf_wCE_eqWE cf_wDE_eqWE cf_wOI_eqWE cf_wWE_eqWE cf_tHO_eqWE cf_tCE_eqWE cf_tDE_eqWE cf_tOI_eqWE cf_tWE_eqWE cf_kHO_eqWE cf_kCE_eqWE cf_kDE_eqWE cf_kOI_eqWE cf_kWE_eqWE cfWE_eqWE mWE_eqWE, nocons)
*       , constraints(2 3 4 6 7 9 12 13 14 15) corr isur iterate(`maxiter');
*       (sHO: yHO = cf_wHO_eqHO cf_wCE_eqHO cf_wDE_eqHO cf_wOI_eqHO cf_wWE_eqHO cf_tHO_eqHO cf_tCE_eqHO cf_tDE_eqHO cf_tOI_eqHO cf_tWE_eqHO cf_kHO_eqHO cf_kCE_eqHO cf_kDE_eqHO cf_kOI_eqHO cf_kWE_eqHO cfHO_eqHO mHO_eqHO, nocons); /* left-out equation */
* };

foreach reg in $regagg {;
  local r2dg = substr("`reg'",1,2);
  predict epsi`r2dg', equation(s`r2dg') residuals;
  };
preserve;
  keep mneid year epsi* iCE* iDE* iOI* iWE*;
  replace mneid = _n;
  rename mneid currno;
  sort currno year;
  save "$rootslct/estmat/v3fs/pt$par-af$aff-$waglvl-cf/byreg/translog`usemissg'-res-$waglvl-pt$par-af$aff", replace;
restore;
drop epsi*;

mat VCE = get(VCE);  /* uncorrected after re-run */
tempname VCEkeep; mat `VCEkeep' = VCE;
local regnum = rowsof(VCE)/e(k_eq);
forvalues i = 1/`e(k_eq)' {;  /* substition of corrected Cov by equation (partition-wise) */
  local frm = (`i'-1)*`regnum' + 1;
  mat VCE[`frm',`frm'] = Vsur`i';
  };
capture RepostSu VCE;
if _rc~=0 {;
 if _rc==506 {;
  disp in blue "Fully corrected SURE matrix not positive definite";
  disp in blue "Independence of nu(k) and eta(l) likely not satisfied in data";
  disp in blue "SURE cross equation tests not well defined";
  local regnum = rowsof(VCE)/e(k_eq);
  mat VCE = J(rowsof(VCE),rowsof(VCE),0);
  forvalues i = 1/`e(k_eq)' {;  /* substition of corrected Cov by equation (partition-wise) */
    local frm = (`i'-1)*`regnum' + 1;
    mat VCE[`frm',`frm'] = Vsur`i';
    };
  RepostSu VCE;
  };
 };
else {;
 RepostSu VCE;
 };
capture sureg;  /* replay w/ Heckman corrected Cov if successful; works only in version 8 and earlier */
est2vec translog`usemissg', e(N rss k_eq levels chi2_bp df_bp r2_1 chi2_1 p_1 r2_2 chi2_2 p_2 r2_3 chi2_3 p_3 r2_4 chi2_4 p_4) name(translog_`usemissg');

if `usemissg' == 0 {;
  order mneid year sHO kHO tHO wHO cfHO sCE kCE tCE wCE cfCE sDE kDE tDE wDE cfDE sOI kOI tOI wOI cfOI sWE kWE tWE wWE cfWE;
  };
if `usemissg' >= 1 {;
  order mneid year sHO kHO tHO wHO nofdiHO cfHO sCE kCE tCE wCE nofdiCE cfCE sDE kDE tDE wDE nofdiDE cfDE sOI kOI tOI wOI nofdiOI cfOI sWE kWE tWE wWE nofdiWE cfWE;
  };


/* initial concavity checks */
/*--------------------------*/

* matrix to check (i) that the fitted cost shares sLOC_predicted are nonnegative;
mat negxhatcnt`usemissg' = J(2,4,.);
matname negxhatcnt`usemissg' r(N_neg_xhare) r(N), rows(1..2) explicit;
local i = 0;
foreach sreg in sCE sDE sOI sWE {;
  local i = `i' + 1;
  matname negxhatcnt`usemissg' `sreg', columns(`i') explicit;
  predict _`sreg' if e(sample), equation(`sreg') xb;
  qui count if _`sreg'<0 & e(sample);
  mat negxhatcnt`usemissg'[1,`i']=`r(N)';
  qui count if e(sample);
  mat negxhatcnt`usemissg'[2,`i']=`r(N)';
  };
drop _s* __*;


/* sample statistics for elasticities of substitution */
/*----------------------------------------------------*/

order mneid year nreg sec1_bem i* _I*;
summ2tex * if e(sample), c(mean sd N p50) name(summ_translog`usemissg') p(tex/v3fs/pt$par-af$aff-$waglvl-cf) replace detail;
preserve;
  foreach var in sHO sCE sDE sOI sWE wHO wCE wDE wOI wWE tHO tCE tDE tOI tWE kHO kCE kDE kOI kWE cfHO cfCE cfDE cfOI cfWE {;
    replace `var' = . if `var'==0;
    };
  summ2tex * if e(sample), c(mean sd N p50) name(summ_translog`usemissg'_missg) p(tex/v3fs/pt$par-af$aff-$waglvl-cf) replace detail;
  foreach reg in $regagg {;
    local r2dg = substr("`reg'",1,2);
    summ2tex * if e(sample) & nofdi`r2dg'==0, c(mean sd N p50) name(summ_translog`usemissg'_missg`r2dg') p(tex/v3fs/pt$par-af$aff-$waglvl-cf) replace detail;
    };
restore;
corr2tex sHO sCE sDE sOI sWE if e(sample), name(corrT_translog`usemissg') p(tex/v3fs/pt$par-af$aff-$waglvl-cf) replace;


/* elasticities of substitution */
/*------------------------------*/

* Cross-wage coefficient matrix;
tempname VCE; /* resets prior `VCE' if defined */
tempname bcoef bse coefmat0 semat0 coefmat1 semat1 coefmat2 semat2 coefmat3 semat3 coefmat4 semat4; 
mat `bcoef' = e(b)';
mat `VCE' = e(V);  /* Heckman corrected if positive semi-definite under re-run */
mat `bse' = vecdiag(`VCE')';
* mat `bse' = vecdiag(`VCEkeep')'; /* Permit checking for symmetry in absence of Heckman correction */
foreach mattyp in coef se {;
  mat ``mattyp'mat0' = J(5,1,.);
  local cnt 0;
  foreach reg in $regagg {;
    local cnt = `cnt' + 1;
    local r2dg = substr("`reg'",1,2);
    if `usemissg'~=2  matorder `b`mattyp'', m(``mattyp'mat`cnt'') ifr(s`r2dg':cf_wHO s`r2dg':cf_wCE s`r2dg':cf_wDE s`r2dg':cf_wOI s`r2dg':cf_wWE);
    if `usemissg'==2  matorder `b`mattyp'', m(``mattyp'mat`cnt'') ifr(s`r2dg':cf_wHO_eq`r2dg' s`r2dg':cf_wCE_eq`r2dg' s`r2dg':cf_wDE_eq`r2dg' s`r2dg':cf_wOI_eq`r2dg' s`r2dg':cf_wWE_eq`r2dg');
    local col`cnt' : roweq ``mattyp'mat`cnt''; local col`cnt' = substr("`col`cnt''",1,3);
    };
  * mat list ``mattyp'mat1';
  * mat list ``mattyp'mat2';
  * mat list ``mattyp'mat3';
  * mat list ``mattyp'mat4';
  mat `mattyp'mat = ``mattyp'mat0' , ``mattyp'mat1' , ``mattyp'mat2' , ``mattyp'mat3' , ``mattyp'mat4';
  * mat list `mattyp'mat;
  mat ``mattyp'mat0' = `mattyp'mat[1,1..5] ';
  mat `mattyp'mat = ``mattyp'mat0' , ``mattyp'mat1' , ``mattyp'mat2' , ``mattyp'mat3' , ``mattyp'mat4';
  mat `mattyp'mat[1,1] = - (`mattyp'mat[2,1] + `mattyp'mat[3,1] + `mattyp'mat[4,1] + `mattyp'mat[5,1]);
  local collbl = subinstr("wHO `col1' `col2' `col3' `col4'","s","w",.);
  local rowlbl = "sHO `col1' `col2' `col3' `col4'";
  * disp "`collbl'";
  * disp "`rowlbl'";
  matname `mattyp'mat `collbl', columns(1..5) explicit;
  matname `mattyp'mat `rowlbl', rows(1..5) explicit;
};
* standard error for sHOwHO;
local sdHOHO = 0;
forvalues i = 2/5 {;
  forvalues j = 2/5 {;
    local sdHOHO = `sdHOHO' + semat[`i',`j'];
    };
  };
mat semat[1,1] = sqrt(`sdHOHO');
mat list coefmat; mat list semat;
tempname chk;
mat `chk' = vecdiag(coefmat); /* check for symmetry */
* if issym(coefmat)~=1 {; * disp as err "matrix coefmat not symmetric"; * STOP; * };

* Observation-specific extensive margins;
foreach reg in HOM $regagg {;
  local r2dg = substr("`reg'",1,2);
  capture local mtrlg`r2dg' = _b[s`r2dg':m`r2dg'];  /* inverse Mill's ratio penultimate regressor in re-run */
  if _rc {;
    local mtrlg`r2dg' = .;
    };
  foreach weff in HOM $regagg {;
    local w2dg = substr("`weff'",1,2);
    local g`r2dg'w`w2dg' = prbdlnw[rownumb(prbdlnw,"i_`reg'"),colnumb(prbdlnw,"w_`weff'")];
    local regvar = "w_othr_`weff'";
    if "`weff'"=="HOM"  local regvar = "w_`weff'";
    * qui gen double xmr`r2dg'w`w2dg' = `g`r2dg'w`w2dg'' * `mtrlg`r2dg'' * dlta`r2dg' * i`r2dg'now / cf`r2dg' ; /* if log wages used */
    qui gen double xmr`r2dg'w`w2dg' = `g`r2dg'w`w2dg'' * `mtrlg`r2dg'' * dlta`r2dg' * i`r2dg'now * `regvar' / cf`r2dg' ; /* if plain wages used */
    replace xmr`r2dg'w`w2dg' = 0 if xmr`r2dg'w`w2dg' == .;
    };
  };

* Matrix of mean extensive margin correction terms;
capture mat drop xmrmat;
local rownames : rowfullnames coefmat;  /* Match regional pattern in coefmat at intensive margin */
local colnames : colfullnames coefmat;
local colnum : word count `colnames';
foreach reg in `rownames' {;
  local r2dg = substr("`reg'",-2,2);
  tempname xmat`r2dg';
  mat `xmat`r2dg'' = J(1,`colnum',.);
  matname `xmat`r2dg'' s`r2dg', rows(1) explicit;
  local colj = 0;
  foreach weff in `colnames' {;
    local colj = `colj' + 1;
    qui summ xmr`r2dg'`weff' if s`r2dg'~=0 & e(sample), meanonly;  /* only calculate mean over sample MNEs with presence */
    mat `xmat`r2dg''[1,`colj'] = r(mean);
    matname `xmat`r2dg'' `weff', columns(`colj') explicit;
    };
  capture mat xmrmat = xmrmat \ `xmat`r2dg'';
  if _rc {;
    mat xmrmat = `xmat`r2dg'';
    };
  };
foreach reg in `rownames' {;
  local r2dg = substr("`reg'",-2,2);
  disp; 
  disp "Coefficient on Mill's ratio in `r2dg': `mtrlg`r2dg''";
  disp "Extensive margin correction terms for `r2dg'";
  summ xmr`r2dg'w* if s`r2dg'~=0 & e(sample);
  };
mat list xmrmat;

* Statistics for matrices of shares (predicted shares recommended by Berndt 1991 and Hayashi 2000 but not used);
local eqs = subinstr("`rownames'","sHO","",.);
foreach reg in `eqs' {;
  local r2dg = substr("`reg'",-2,2);
  * qui predict double shat`r2dg', equation(s`r2dg') xb, if e(sample);
  gen double shat`r2dg' = s`r2dg';
  };
gen double shatHO = sHO;
preserve;
collapse (mean) shatHO shatCE shatDE shatOI shatWE if e(sample), fast;
mkmat shatHO shatCE shatDE shatOI shatWE, matrix(Tmat);  /* Tmat to be transposed to a column vector below */
matname Tmat theta, rows(1) explicit;
restore, preserve;
collapse (sd) shatHO shatCE shatDE shatOI shatWE if e(sample), fast;
mkmat shatHO shatCE shatDE shatOI shatWE, matrix(Tmat_sd);
matname Tmat_sd theta_sd, rows(1) explicit;
restore, preserve;
collapse (count) shatHO shatCE shatDE shatOI shatWE if e(sample), fast;
mkmat shatHO shatCE shatDE shatOI shatWE, matrix(Tmat_N);
matname Tmat_N theta_N, rows(1) explicit;
restore, preserve;

* Zeta matrix (r2 in Anderson and Thursby 1986);
foreach sreg in HOM $regagg {;
  local sr2dg = substr("`sreg'",1,2);
  foreach wreg in HOM $regagg {;
    local wr2dg = substr("`wreg'",1,2);
    qui gen double r2s`sr2dg's`wr2dg' = Tmat_N[1,colnumb(Tmat_N,"s`sr2dg'")]^(-3) * ( ( s`sr2dg' - Tmat[1,colnumb(Tmat,"s`sr2dg'")] ) * ( s`sr2dg'*s`wr2dg' - Tmat[1,colnumb(Tmat,"s`sr2dg'")]*Tmat[1,colnumb(Tmat,"s`wr2dg'")] - corrT_translog`usemissg'_tbl[rownumb(corrT_translog`usemissg'_tbl,"s`sr2dg'"),colnumb(corrT_translog`usemissg'_tbl,"s`wr2dg'")]*Tmat_sd[1,colnumb(Tmat_sd,"s`sr2dg'")]*Tmat_sd[1,colnumb(Tmat_sd,"s`wr2dg'")] ) + Tmat_N[1,colnumb(Tmat_N,"s`sr2dg'")]*(Tmat_N[1,colnumb(Tmat_N,"s`sr2dg'")] - 1)*Tmat[1,colnumb(Tmat,"s`sr2dg'")]*Tmat_sd[1,colnumb(Tmat_sd,"s`sr2dg'")]*( Tmat_sd[1,colnumb(Tmat_sd,"s`sr2dg'")] + corrT_translog`usemissg'_tbl[rownumb(corrT_translog`usemissg'_tbl,"s`sr2dg'"),colnumb(corrT_translog`usemissg'_tbl,"s`wr2dg'")]*Tmat_sd[1,colnumb(Tmat_sd,"s`wr2dg'")] ) );
    };
  };
collapse (sum) r2sHOsHO r2sHOsCE r2sHOsDE r2sHOsOI r2sHOsWE r2sCEsHO r2sCEsCE r2sCEsDE r2sCEsOI r2sCEsWE r2sDEsHO r2sDEsCE r2sDEsDE r2sDEsOI r2sDEsWE r2sOIsHO r2sOIsCE r2sOIsDE r2sOIsOI r2sOIsWE r2sWEsHO r2sWEsCE r2sWEsDE r2sWEsOI r2sWEsWE if e(sample), fast;
tempname zetatmp;
mkmat r2sHOsHO r2sHOsCE r2sHOsDE r2sHOsOI r2sHOsWE r2sCEsHO r2sCEsCE r2sCEsDE r2sCEsOI r2sCEsWE r2sDEsHO r2sDEsCE r2sDEsDE r2sDEsOI r2sDEsWE r2sOIsHO r2sOIsCE r2sOIsDE r2sOIsOI r2sOIsWE r2sWEsHO r2sWEsCE r2sWEsDE r2sWEsOI r2sWEsWE, matrix(`zetatmp');
local shnames : colfullnames Tmat;
local shnames = subinstr("`shnames'","hat","",.);
mat zetamat = J(5,5,.);
matname zetamat `shnames', rows(1..5) explicit;
matname zetamat `shnames', columns(1..5) explicit;
foreach sr in `shnames' {;
  local sr2dg = subinstr("`sr'","s","",.);
  foreach wr in `shnames' {;
    local wr2dg = subinstr("`wr'","s","",.);
    mat zetamat[rownumb(zetamat,"`sr'"),colnumb(zetamat,"`wr'")] = `zetatmp'[1,colnumb(`zetatmp',"r2s`sr2dg's`wr2dg'")];
    };
  };
restore;

* Matrices of shares;
mat Tmat_all = Tmat \ Tmat_sd \ Tmat_N;
mat Tmat = Tmat'; /* Tmat transposed to a column vector */
mat Tsqmat = Tmat * Tmat';

* Calculate total cross-price elasticities (both margins);
mat TotElast = coefmat + xmrmat + Tsqmat;
forvalues r = 2/5 {; /* Fill lower-echelon entries */
local maxcol = `r'-1;
  forvalues m = 1/`maxcol' {;
    mat TotElast[`r',`m'] = TotElast[`r',`m'] / Tmat[`r',1];
    };
  };
forvalues m = 2/5 {; /* Fill upper-echelon entries */
local maxrow = `m'-1;
  forvalues r = 1/`maxrow' {;
    mat TotElast[`r',`m'] = TotElast[`r',`m'] / Tmat[`r',1];
    };
  };
forvalues r = 1/5 {;  /* Fill diagonal entries */
    mat TotElast[`r',`r'] = TotElast[`r',`r'] / Tmat[`r',1] - 1;
  };
matname TotElast `shnames', rows(1..5) explicit;
matname TotElast `shnames', columns(1..5) explicit;
mat list TotElast;
disp;

* Calculate cross-price elasticities matrix at extensive margin;
mat ExtElast = xmrmat + Tsqmat;
forvalues r = 2/5 {; /* Fill lower-echelon entries */
local maxcol = `r'-1;
  forvalues m = 1/`maxcol' {;
    mat ExtElast[`r',`m'] = ExtElast[`r',`m'] / Tmat[`r',1];
    };
  };
forvalues m = 2/5 {; /* Fill upper-echelon entries */
local maxrow = `m'-1;
  forvalues r = 1/`maxrow' {;
    mat ExtElast[`r',`m'] = ExtElast[`r',`m'] / Tmat[`r',1];
    };
  };
forvalues r = 1/5 {;  /* Fill diagonal entries and cancel HOM row */
    mat ExtElast[`r',`r'] = ExtElast[`r',`r'] / Tmat[`r',1] - 1;
    mat ExtElast[1,`r'] = 0;
  };
matname ExtElast `shnames', rows(1..5) explicit;
matname ExtElast `shnames', columns(1..5) explicit;
mat list ExtElast;
disp;

* Calculate cross-price elasticities matrix at intensive margin;
mat IntElast = coefmat + Tsqmat;
forvalues r = 2/5 {; /* Fill lower-echelon entries */
local maxcol = `r'-1;
  forvalues m = 1/`maxcol' {;
    mat IntElast[`r',`m'] = IntElast[`r',`m'] / Tmat[`r',1];
    };
  };
forvalues m = 2/5 {; /* Fill upper-echelon entries */
local maxrow = `m'-1;
  forvalues r = 1/`maxrow' {;
    mat IntElast[`r',`m'] = IntElast[`r',`m'] / Tmat[`r',1];
    };
  };
forvalues r = 1/5 {;  /* Fill diagonal entries */
    mat IntElast[`r',`r'] = IntElast[`r',`r'] / Tmat[`r',1] - 1;
  };
matname IntElast `shnames', rows(1..5) explicit;
matname IntElast `shnames', columns(1..5) explicit;
mat list IntElast;
disp;


/* additional concavity checks */
/*-----------------------------*/

* matrix to check (ii) that the cross price elasticities in TotElast sum to zero per row (not per column);
mat elastsum`usemissg' = J(2,5,0);
matname elastsum`usemissg' rowsum colsum, rows(1..2) explicit;
matname elastsum`usemissg' sHO sCE sDE sOI sWE, columns(1..5) explicit;
local i = 0;
foreach sreg in sHO sCE sDE sOI sWE {;
  local i = `i' + 1;
  forvalues j = 1/5 {;
    mat elastsum`usemissg'[1,`i'] = elastsum`usemissg'[1,`i'] + TotElast[`i',`j'];
    mat elastsum`usemissg'[2,`i'] = elastsum`usemissg'[2,`i'] + TotElast[`j',`i'];
    };
  };

* matrix to check (iii) that the AUmat matrices of Allen-Uzawa substitution elasticities are negative semi-definite at each observation;
* calculate MNE-level elasticities (in-sample predictions); 
* set elasticities to zero for regions with no FDI-presence;
local i = 0;
foreach reg1 in HO CE DE OI WE {;
  local i = `i' + 1;
  local j = 0;
  foreach reg2 in HO CE DE OI WE {;
    local j = `j' + 1;
    local coeff = coefmat[`i',`j'];
    if `i' ~= `j' {; 
      qui gen double sigma`reg1's`reg2' = (`coeff' + s`reg1' * s`reg2') / (s`reg1' * s`reg2') if e(sample);
      };
    if `i' == `j' {; 
      qui gen double sigma`reg1's`reg1' = (`coeff' + (s`reg1')^2 - s`reg1') / (s`reg1')^2 if e(sample);
      };      
    qui replace sigma`reg1's`reg2'=0 if sigma`reg1's`reg2'==.;
    label var sigma`reg1's`reg2' "AU elast. empl`reg1' empl`reg2'";
    };
  };
qui count;
local obsnum = `r(N)';
tempname AUmatobs eigenvec eigenval;
mat negeivalcnt`usemissg' = J(1,5,0) \ J(1,5,`obsnum');
matname negeivalcnt`usemissg' r(N_wklyneg_eigval) r(N), rows(1..2) explicit;
matname negeivalcnt`usemissg' wklyneg_1 wklyneg_2 wklyneg_3 wklyneg_4 wklyneg_5, columns(1..5) explicit;
forvalues obs = 1/`obsnum'{;
  mat `AUmatobs' = J(5,5,.);
  local i = 0;
  foreach reg1 in HO CE DE OI WE {;
    local i = `i' + 1;
    local j = 0;
    foreach reg2 in HO CE DE OI WE {;
      local j = `j' + 1;
      mat `AUmatobs'[`i',`j'] = sigma`reg1's`reg2'[`obs'];
      };
    };
  capture matrix symeigen `eigenvec' `eigenval' = `AUmatobs';
  if _rc == 0 {;
    matname `eigenval' `eigenval'ue, rows(1) explicit;
    mat drop `eigenvec';
    local cntwklyneg = 0;
    forvalues k = 1/5 {;
      if `eigenval'[1,`k'] <= 1.0e-06 {;  /* machine-size weakly negative */
        local cntwklyneg = `cntwklyneg' + 1;
        };
      };
    mat negeivalcnt`usemissg'[1,`cntwklyneg'] = negeivalcnt`usemissg'[1,`cntwklyneg']+1;
    };
  };


/* matrix output */
/*---------------*/

foreach reg in $regagg {;
  local r2dg = substr("`reg'",1,2);
  foreach type in b se e {;
    drop _all;
    matsave dprobit`r2dg'_`type', p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/byreg) t(double) replace;
    drop _all;
    matsave probit`r2dg'_`type', p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/byreg) t(double) replace;
    drop _all;
    matsave sure2stp`r2dg'_`type', p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/byreg) t(double) replace;
    };
  drop _all;
  matsave probit`r2dg'_r, p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/byreg) t(double) replace;
  };

foreach type in b se e {;
    drop _all;
    matsave translog`usemissg'_`type', p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') t(double) replace;
    };
matsave negxhatcnt`usemissg', p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace; 
matsave elastsum`usemissg', p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace; 
matsave negeivalcnt`usemissg', p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace; 

mat intelast`usemissg'_b = IntElast;
matsave intelast`usemissg'_b, p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace;
mat extelast`usemissg'_b = ExtElast;
matsave extelast`usemissg'_b, p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace;
mat totelast`usemissg'_b = TotElast;
matsave totelast`usemissg'_b, p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace;

mat vce`usemissg'_tbl = `VCE';
local cols = colsof(vce`usemissg'_tbl);
matsave vce`usemissg'_tbl, p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace;

mat tmat`usemissg'_tbl = Tmat_all;
matsave tmat`usemissg'_tbl, p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace;
mat zetamat`usemissg'_tbl = zetamat;
matsave zetamat`usemissg'_tbl, p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace;

mat coefmat`usemissg'_tbl = coefmat;
matname coefmat`usemissg'_tbl wHO wCE wDE wOI wWE, columns(1..5) explicit;
matname coefmat`usemissg'_tbl sHO sCE sDE sOI sWE, rows(1..5) explicit;
matsave coefmat`usemissg'_tbl, p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace;

mat semat`usemissg'_tbl = semat;
matname semat`usemissg'_tbl wHO wCE wDE wOI wWE, columns(1..5) explicit;
matname semat`usemissg'_tbl sHO sCE sDE sOI sWE, rows(1..5) explicit;
matsave semat`usemissg'_tbl, p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace;

mat xmrmat`usemissg'_tbl = xmrmat;
matname xmrmat`usemissg'_tbl wHO wCE wDE wOI wWE, columns(1..5) explicit;
matname xmrmat`usemissg'_tbl sHO sCE sDE sOI sWE, rows(1..5) explicit;
matsave xmrmat`usemissg'_tbl, p(estmat/v3fs/pt$par-af$aff-$waglvl-cf/translog`usemissg') dropall t(double) replace;

* clean up;
drop _all;
mat drop coefmat semat xmrmat zetamat Tmat Tmat_sd Tmat_N Tmat_all Tsqmat TotElast ExtElast IntElast;
foreach mat in summ_translog`usemissg' summ_translog`usemissg'_missg corrT_translog`usemissg' vce`usemissg' coefmat`usemissg' semat`usemissg' tmat`usemissg' xmrmat`usemissg' zetamat`usemissg' {;
  mat drop `mat'_tbl;
  };
foreach type in b se e r {;
  capture mat drop translog`usemissg'_`type';
  foreach reg in $regagg {;
    local r2dg = substr("`reg'",1,2);
    capture mat drop dprobit`r2dg'_`type';
    capture mat drop probit`r2dg'_`type';
    capture mat drop sure2stp`r2dg'_`type';
    };
  };
foreach reg in $regagg {;
  local r2dg = substr("`reg'",1,2);
  mat drop bprb`r2dg' Vprb`r2dg';
  };
mat drop negxhatcnt`usemissg' elastsum`usemissg' negeivalcnt`usemissg';
mat drop totelast`usemissg'_b;
mat drop intelast`usemissg'_b;
mat drop extelast`usemissg'_b;

}; /* end of `usemissg' loop */

capture program drop Heck2eqs;
capture program drop RepostSu;
capture program drop Display2;

*----------------------above code identical to translog-selct-2stp-cf.do----------------------------------------------------;

log close;
cd "$root";
