*! version 1.0.4  13nov2006  by Marc Muendler  muendler@ucsd.edu

clear
set memory 400m
set matsize 400
set more off
capture log close

cd "$rootslct"

log using "logs/comp-elast-ger-cf.log", replace

capture mkdir "$rootslct/tex/comp"
capture mkdir "$rootslct/tex/comp/pt$par-af$aff-cf"

mat drop _all

adopath + "$root/%stata/ado"

local wag = subinstr(subinstr("$waglvl","frm","",.),"wage","",.)
if "`wag'"=="uno"  local othrwag = "oww"
if "`wag'"=="oww"  local othrwag = "uno"

* COMPARISON MATRICES OF ELASTICITIES

foreach mat in cmpintw cmpextw cmpinty {
  capture est2extlbl, addto(`mat') retain p(tex/comp/pt$par-af$aff-cf) replace
  if _rc~=0  est2extlbl, addto(`mat') intex($rootdo/%stata/rowlbl-fdiselct) p(tex/comp/pt$par-af$aff-cf) replace
  }
clear

* home wage effects on foreign employment (first column of elasticity matrices)
foreach mat in int ext {
  matload cmp`mat'elastw_tbl, p(tex/v3npfs/pt$par-af$aff-frm`wag'wage-cf) over missing(.)
  matrix coleq cmp`mat'elastw_tbl = fs`wag'
  mat rename cmp`mat'elastw_tbl cmp`mat'w_tbl
  matload cmp`mat'elastw_tbl, p(tex/v3npfs/pt$par-af$aff-frm`othrwag'wage-cf) over missing(.)
  matrix coleq cmp`mat'elastw_tbl = fs`othrwag'
  mat cmp`mat'w_tbl = cmp`mat'w_tbl , cmp`mat'elastw_tbl
  matload cmp`mat'elastw_tbl, p(tex/v3np/pt$par-af$aff-frm`wag'wage-cf) over missing(.)
  matrix coleq cmp`mat'elastw_tbl = cr`wag'
  mat cmp`mat'w_tbl = cmp`mat'w_tbl , cmp`mat'elastw_tbl
  mat drop cmp`mat'elastw_tbl
  local colnames : colnames cmp`mat'w_tbl
  tokenize `colnames'
  local num : word count `colnames'
  forvalues i = 1/`num' {
    local newname = subinstr("``i''","_mWageHO","",.)
    matname cmp`mat'w_tbl `newname', column(`i') explicit
    }
  matorder cmp`mat'w_tbl, m(cmp`mat'w_tbl) over ifc(fs`wag':wHO1hm cr`wag':wHO1hm fs`othrwag':wHO1hm fs`wag':wHO1np fs`wag':wHO1npr fs`wag':wHO0hm fs`wag':wHO2hm)
  est2tex cmp`mat'w, replace dropall preserve mark(starse) path(tex/comp/pt$par-af$aff-cf) fancy levels(90 95 99) ready
  }

* foreign wage effects on home employment (first row of elasticity matrices)
local mat "int"
  matload cmp`mat'elasty_tbl, p(tex/v3npfs/pt$par-af$aff-frm`wag'wage-cf) over missing(.)
  local roweq : roweq cmp`mat'elasty_tbl
  local rows = rowsof(cmp`mat'elasty_tbl)
  tokenize `roweq'
  local newrows ""
  forvalues i = 1/`rows' {
    local newrows "`newrows' fs`wag'_``i''"
    }
  matrix roweq cmp`mat'elasty_tbl = `newrows'
  mat rename cmp`mat'elasty_tbl cmp`mat'y_tbl
  matload cmp`mat'elasty_tbl, p(tex/v3npfs/pt$par-af$aff-frm`othrwag'wage-cf) over missing(.)
  local roweq : roweq cmp`mat'elasty_tbl
  local rows = rowsof(cmp`mat'elasty_tbl)
  tokenize `roweq'
  local newrows ""
  forvalues i = 1/`rows' {
    local newrows "`newrows' fs`othrwag'_``i''"
    }
  matrix roweq cmp`mat'elasty_tbl = `newrows'
  mat cmp`mat'y_tbl = cmp`mat'y_tbl \ cmp`mat'elasty_tbl
  matload cmp`mat'elasty_tbl, p(tex/v3np/pt$par-af$aff-frm`wag'wage-cf) over missing(.)
  local roweq : roweq cmp`mat'elasty_tbl
  local rows = rowsof(cmp`mat'elasty_tbl)
  tokenize `roweq'
  local newrows ""
  forvalues i = 1/`rows' {
    local newrows "`newrows' cr`wag'_``i''"
    }
  matrix roweq cmp`mat'elasty_tbl = `newrows'
  mat cmp`mat'y_tbl = cmp`mat'y_tbl \ cmp`mat'elasty_tbl
  matorder cmp`mat'y_tbl, m(cmp`mat'y_tbl) over ifr(fs`wag'_yHO1hm:rLDemHO fs`wag'_yHO1hm:_se cr`wag'_yHO1hm:rLDemHO cr`wag'_yHO1hm:_se fs`othrwag'_yHO1hm:rLDemHO fs`othrwag'_yHO1hm:_se fs`wag'_yHO1np:rLDemHO fs`wag'_yHO1np:_se fs`wag'_yHO1npr:rLDemHO fs`wag'_yHO1npr:_se fs`wag'_yHO0hm:rLDemHO fs`wag'_yHO0hm:_se fs`wag'_yHO2hm:rLDemHO fs`wag'_yHO2hm:_se)
  est2tex cmp`mat'y, replace dropall preserve mark(starse) path(tex/comp/pt$par-af$aff-cf) fancy levels(90 95 99) ready

log close
cd "$root"
