program multiheck_d1

version 8.1
args todo b lnf g negH scb1 scb2 scb3 scb4 scg1 scg2 scg3 scg4 scs1 scs2 scs3 scs4 scp1 scp2 scp3 scp4

*** STEP 1:  Define the log-likelihood.

	*** Define Beta and Gamma vectors;
	tempvar Ir1 Ir2 Ir3 Ir4 Is1 Is2 Is3 Is4
	mleval `Ir1'  = `b', eq(1)
	mleval `Is1'  = `b', eq(2)
	mleval `Ir2'  = `b', eq(3)
	mleval `Is2'  = `b', eq(4)
	mleval `Ir3'  = `b', eq(5)
	mleval `Is3'  = `b', eq(6)
	mleval `Ir4'  = `b', eq(7)
	mleval `Is4'  = `b', eq(8)
	
	*** Define Sigma-eta matrix;	
	tempname lns1 lns2 lns3 lns4 sig1 sig2 sig3 sig4
	mleval `lns1' = `b', scalar eq(9)
	mleval `lns2' = `b', scalar eq(10)
	mleval `lns3' = `b', scalar eq(11)
	mleval `lns4' = `b', scalar eq(12)

	scalar `sig1'  = exp(`lns1')
	scalar `sig2'  = exp(`lns2')
	scalar `sig3'  = exp(`lns3')
	scalar `sig4'  = exp(`lns4')

	*** Define Rhos;
	tempname tau1 tau2 tau3 tau4 rho1 rho2 rho3 rho4
	mleval `tau1' = `b', scalar eq(13)
	mleval `tau2' = `b', scalar eq(14)
	mleval `tau3' = `b', scalar eq(15)
	mleval `tau4' = `b', scalar eq(16)

      scalar `rho1'  = (exp(2*`tau1')-1)  /  (exp(2*`tau1')+1)
      scalar `rho2'  = (exp(2*`tau2')-1)  /  (exp(2*`tau2')+1)
      scalar `rho3'  = (exp(2*`tau3')-1)  /  (exp(2*`tau3')+1)
      scalar `rho4'  = (exp(2*`tau4')-1)  /  (exp(2*`tau4')+1)
	
	*** Define Selection Correction Terms;
	tempvar eta1 eta2 eta3 eta4 
	qui gen double `eta1' = (`Is1' +  ($ML_y1 -`Ir1')*`rho1'/`sig1') / sqrt(1-`rho1'^2)
	qui gen double `eta2' = (`Is2' +  ($ML_y3 -`Ir2')*`rho2'/`sig2') / sqrt(1-`rho2'^2)
	qui gen double `eta3' = (`Is3' +  ($ML_y5 -`Ir3')*`rho3'/`sig3') / sqrt(1-`rho3'^2)
	qui gen double `eta4' = (`Is4' +  ($ML_y7 -`Ir4')*`rho4'/`sig4') / sqrt(1-`rho4'^2)
	
	mlsum `lnf' = cond($ML_y2,(ln(normprob(`eta1')) - 0.5*ln(2*_pi*`sig1'^2) - 0.5*(($ML_y1-`Ir1')/`sig1')^2), ln(normprob(-`Is1'))) //
		//	+ cond($ML_y4,(ln(normprob(`eta2')) - 0.5*ln(2*_pi*`sig2'^2) - 0.5*(($ML_y3-`Ir2')/`sig2')^2), ln(normprob(-`Is2'))) //
		//	+ cond($ML_y6,(ln(normprob(`eta3')) - 0.5*ln(2*_pi*`sig3'^2) - 0.5*(($ML_y5-`Ir3')/`sig3')^2), ln(normprob(-`Is3'))) //
		//	+ cond($ML_y8,(ln(normprob(`eta4')) - 0.5*ln(2*_pi*`sig4'^2) - 0.5*(($ML_y7-`Ir4')/`sig4')^2), ln(normprob(-`Is4')))

if (`todo'==0 | `lnf' >=.) exit

*** STEP 2:  Define the gradient vector.

	tempvar M1 M2 M3 M4 ym1 ym2 ym3 ym4 Mp1 Mp2 Mp3 Mp4 
	tempname rr1 rr2 rr3 rr4 dlr1 dlr2 dlr3 dlr4

	** Define Inverse Mills Ratios for each set of equations;
	qui gen double `M1' = normd(`eta1') / normprob(`eta1')
	qui gen double `M2' = normd(`eta2') / normprob(`eta2')
	qui gen double `M3' = normd(`eta3') / normprob(`eta3')
	qui gen double `M4' = normd(`eta4') / normprob(`eta4')

	** Define [(x-y'b)/sigma] for each share equation;
	qui gen double `ym1' = ($ML_y1-`Ir1') / `sig1'
	qui gen double `ym2' = ($ML_y3-`Ir2') / `sig2'
	qui gen double `ym3' = ($ML_y5-`Ir3') / `sig3'
	qui gen double `ym4' = ($ML_y7-`Ir4') / `sig4'

	** Drop (y'b) for each share equation;
	drop `Ir1' `Ir2' `Ir3' `Ir4'
	
	** Define 1/sqrt(1-rho2);
	scalar `rr1' = 1/sqrt(1-`rho1'^2)
	scalar `rr2' = 1/sqrt(1-`rho2'^2)
	scalar `rr3' = 1/sqrt(1-`rho3'^2)
	scalar `rr4' = 1/sqrt(1-`rho4'^2)
	
	** Define 
	qui gen double `Mp1' = normd(-(`Is1')) / normprob(-(`Is1'))
	qui gen double `Mp2' = normd(-(`Is2')) / normprob(-(`Is2'))
	qui gen double `Mp3' = normd(-(`Is3')) / normprob(-(`Is3'))
	qui gen double `Mp4' = normd(-(`Is4')) / normprob(-(`Is4'))

	** Define dRho/dTau;
	scalar `dlr1'  = 4*exp(2*`tau1') / ((exp(2*`tau1')+1)^2)
	scalar `dlr2'  = 4*exp(2*`tau2') / ((exp(2*`tau2')+1)^2)
	scalar `dlr3'  = 4*exp(2*`tau3') / ((exp(2*`tau3')+1)^2)
	scalar `dlr4'  = 4*exp(2*`tau4') / ((exp(2*`tau4')+1)^2)

	** Define dlnL/dBeta;
	qui replace `scb1' = cond($ML_y2,`ym1'/`sig1'-`M1'*`rho1'/`sig1'*`rr1',0)
	qui replace `scb2' = cond($ML_y4,`ym2'/`sig2'-`M2'*`rho2'/`sig2'*`rr2',0)
	qui replace `scb3' = cond($ML_y6,`ym3'/`sig3'-`M3'*`rho3'/`sig3'*`rr3',0)
	qui replace `scb4' = cond($ML_y8,`ym4'/`sig4'-`M4'*`rho4'/`sig4'*`rr4',0)

	** Define dlnL/dGamma;
	qui replace `scg1' = cond($ML_y2,`M1'*`rr1' , -`Mp1')
	qui replace `scg2' = cond($ML_y4,`M2'*`rr2' , -`Mp2')
	qui replace `scg3' = cond($ML_y6,`M3'*`rr3' , -`Mp3')
	qui replace `scg4' = cond($ML_y8,`M4'*`rr4' , -`Mp4')

	** Define dlnL/dSigma-eta;
	qui replace `scs1' = cond($ML_y2,-`M1'*`ym1'*`rr1'*`rho1'+`ym1'*`ym1'-1,0)
	qui replace `scs2' = cond($ML_y4,-`M2'*`ym2'*`rr2'*`rho2'+`ym2'*`ym2'-1,0)
	qui replace `scs3' = cond($ML_y6,-`M3'*`ym3'*`rr3'*`rho3'+`ym3'*`ym3'-1,0)
	qui replace `scs4' = cond($ML_y8,-`M4'*`ym4'*`rr4'*`rho4'+`ym4'*`ym4'-1,0)
	
	** Define dlnL/dRho;
	qui replace `scp1' = cond($ML_y2,`M1'*(`ym1'*`rr1'+`eta1'*`rho1'*`rr1'^2)*`dlr1',0)
	qui replace `scp2' = cond($ML_y4,`M2'*(`ym2'*`rr2'+`eta2'*`rho2'*`rr2'^2)*`dlr2',0)
	qui replace `scp3' = cond($ML_y6,`M3'*(`ym3'*`rr3'+`eta3'*`rho3'*`rr3'^2)*`dlr3',0)
	qui replace `scp4' = cond($ML_y8,`M4'*(`ym4'*`rr4'+`eta4'*`rho4'*`rr4'^2)*`dlr4',0)

	** Define Individual Parameter Gradient Vectors;
	tempname gb1 gb2 gb3 gb4 gb gg1 gg2 gg3 gg4 gg gs1 gs2 gs3 gs4 gs gp1 gp2 gp3 gp4 gp

	** Beta;
	mlvecsum `lnf' `gb1' = `scb1'  if $ML_y2, eq(1)
	mlvecsum `lnf' `gb2' = `scb2'  if $ML_y4, eq(3)
	mlvecsum `lnf' `gb3' = `scb3'  if $ML_y6, eq(5)
	mlvecsum `lnf' `gb4' = `scb4'  if $ML_y8, eq(7)

	matrix `gb' = (`gb1',`gb2',`gb3',`gb4')

	** Gamma;
	mlvecsum `lnf' `gg1' = `scg1',            eq(2)
	mlvecsum `lnf' `gg2' = `scg2',            eq(4)
	mlvecsum `lnf' `gg3' = `scg3',            eq(6)
	mlvecsum `lnf' `gg4' = `scg4',            eq(8)

	matrix `gg' = (`gg1',`gg2',`gg3',`gg4')

	** Sigma-eta;
	mlvecsum `lnf' `gs1' = `scs1'  if $ML_y2, eq(9)
	mlvecsum `lnf' `gs2' = `scs2'  if $ML_y4, eq(10)
	mlvecsum `lnf' `gs3' = `scs3'  if $ML_y6, eq(11)
	mlvecsum `lnf' `gs4' = `scs4'  if $ML_y8, eq(12)

	matrix `gs' = (`gs1',`gs2',`gs3',`gs4')

	** Rho;	
	mlvecsum `lnf' `gp1' = `scp1'  if $ML_y2, eq(13)
	mlvecsum `lnf' `gp2' = `scp2'  if $ML_y4, eq(14)
	mlvecsum `lnf' `gp3' = `scp3'  if $ML_y6, eq(15)
	mlvecsum `lnf' `gp4' = `scp4'  if $ML_y8, eq(16)

	matrix `gp' = (`gp1',`gp2',`gp3',`gp4')

	** Define Matrix of Gradient Vectors;

	matrix `g' = (`gb',`gg',`gs',`gp')

if (`todo'==1 | `lnf' >=.) exit

*** STEP 3:  Define the Hessian.

end
exit
